!     Last change:  RHG  24 Aug 2005   10:48 am
!   caps version 3    7/30/02
!     this version adds fat-tailed distributions
!     (Cauchy) to the dispersal options
!     seed rain algorithm fixed (in regrow) 11/10/04
!   caps version 2
!     previously named 'idh' to examine the intermediate
!     disturbance hypothesis. This version improved the
!     disturbance function of CAPS
!......................................................
!
!   program for spatial simulation of sessile organisms
! --revision of CAPS to handle annual and perenial plants
!   written in FORTRAN 95
!   Robert H. Gardner
!   Appalachian Environmental Laboratory
!   Gunter Hall
!   Frostburg, MD  21532
!
!......................................................
       program capsv3    
       USE repsblock
       USE mapblock
       USE specblock

!  include graphics libraries   -- LINUX VERSION
!
!  filename
!  defect
!  done 
!  idf -- disturbance frequency
       implicit none
       integer :: ik
       logical :: done, speciesOK
       CHARACTER*2 :: pctl

!      open (10, file='saslog.dat', status='unknown', position='append')
       open (15, file='caps.log',   status='unknown')
       open (16, file='contagn.dat',status='unknown')
       open (17, FILE='abund.dat', STATUS='unknown')
       open (18, file='distsum.dat',status='unknown')
!
!  obtain input parameters for model simulation
       call Input
!  dynamic allocation of core
       call SizeIt

!  change parameter loop
       pctl = 'GO'
       do WHILE (pctl .NE. 'ST')

!  initialize statistical arrays
         call StatInit
!  Write label on output data set
         Call WriteLabel
!   begin simulations
         do irep = 1, nrep
!
!   generate random map of irow and jcol dimensions with
!      'ihab' number of habitats
            Call GenMap
!   compute competition among species
            Call CompCoef

!  Before simulations begin, calculate frequency and 
!    times for disturbance -- set up so that the total
!    area disturbed remains the same for all simulations
!    Use white noise, fractal Brownian motion, etc.
            CALL Disturb_setup

!   initialize species distributions
            Call InitSpc

            ik = 0

            done = .false.
            Call WriteData (0, done)

!  begin simulations
            ik = 1
            speciesOK = .true.
            do while (speciesOK .and. ik .lt. simlen)
!
!  simulate a single time step
               call regrow (ik)
!
!  check on species disturbance
               call DisturbIt (ik)
!
!  output the number of species present
               Call WriteData (ik, done)

!  check for species extinctions
               CALL Abun_chk (ik, speciesOK)
               ik = ik+1
            enddo  ! do while ik (year loop)
!
!  irep simulation -- output map and accumulate statistics
            if (gans .eq. 'Y') Call OutputMap (13)
            Call AccumStats (ik)

         enddo     ! do nrep -- map loop

!  output final statistics
         Call StatsOut
!  check for additional simulations
         CALL Prog_Cntrl (pctl)
       END do ! while pclt .ne. 'ST'

!  close files and deallocate core
!      Close(10)    ! saslog.dat
       Close(11)
       CLOSE(13)
       Close(15)
       CLOSE(16)
       CLOSE(17)
       Close(18)
       call DownSize
!
       Stop
       End
!
       Subroutine CompCoef 
!
       USE repsblock
       USE mapblock
       USE specblock
!
!  ft  -- frequency count of habitat types
!  spt -- species competition coefficient =
!           optima * habitat frequency * fecundity
!  cmat-- matrix of relative competition calculations =
!           spt(i) / spt(j)
       real ft(0:maxsp), spt(maxsp), cmat(maxsp)
!
!  count habitat frequencies
       ft = 0.0
       do j = 2, jcol+1
          do i = 2, irow+1
             ij = map_hab(i,j)
             ft(ij) = ft(ij) + 1.0
          enddo
       enddo
!
!  normalize counts
       psum = 0.0
       do i = 0, ihab
          ft(i) = ft(i) / float(irow*jcol)
          psum = psum + ft(i)
       enddo
!
!  calculate competition ratios as a average seed production weighted
!   by habitat frequency
       do i = 1,ispecies
          spt(i) = 0.0
          do j = 1, ihab
             spt(i) = spt(i) + iopt(i,j) * ft(j) * fecund(i)
          enddo
       enddo
!
!  output results competition matrix
!
       print 100, irep
!       write (15,100) irep
100    format (t10,'Map',i3)
      print 101, (i, ft(i), i=0,ihab)
!      write (15,101) (i, ft(i), i=0,ihab)
      print 99, psum
!      write (15,99) psum
 99    format (t15,'Total',t30,g12.6)
101    format (t15,'Habitat ',i1,':      ',g12.6)
!  calculate competition ratios (row species) / (column species)
       PRINT 103, (i, i=1,ispecies)
!       write (15,103) (i, i=1,ispecies)
103    format (t10,'Competition Ratios (Row Species) / (Column Species)', &
               (/t12,8(i4,5x)))
       do i = 1, ispecies
          do j = 1, ispecies
             if (spt(i) .le. 0.0 .or. spt(j) .le. 0.0) then
                cmat(j) = 0.0
              else
                cmat(j) = spt(i) / spt(j)
             endif
          end do
          PRINT 102, i,(cmat(j), j=1,ispecies)
!          write (15,102) i,(cmat(j), j=1,ispecies)
102       format (t8,i2,(t10,8(1x,f8.4)))
       end do
!
       return
       end

       subroutine Abun_chk (ik, speciesOK)

       USE mapblock
       USE repsblock
       USE specblock
       USE dstrb_blk

       implicit none
       INTEGER ispcnt(0:maxsp), i, j, isp_rich, ik
       LOGICAL speciesOK
       REAL rmapdimen

       ispcnt = 0
       do j = 2,jcol+1
          do i = 2, irow+1
             ispcnt(map_sp(i,j)) = ispcnt(map_sp(i,j)) + 1
          end do
       end do

!  check for extinction
       speciesOK = .false.
       isp_rich = 0
       rmapdimen = float(irow*jcol)
       pb = 0.0
       do i = 1, ispecies
          if (ispcnt(i) .gt. 0) isp_rich = isp_rich + 1
          vicdf(i) = float(ispcnt(i))  / rmapdimen 
          pb = pb + vicdf(i)
       end do
       if (isp_rich .gt. 1) speciesOK = .true.
       if (exans .eq. 'n' .or. exans .eq. 'N') speciesOK = .true.

!  normalize abundances to map sizes, multiply by 100 and print percents
       write (17,100) irep, ik, pb, idefect(ik), isp_rich, &
                      (vicdf(i),i=1,ispecies)
100    format (i3,1x,i6,1x,f6.4,2(1x,i6),50(1x,f6.4))
!
       return
       end subroutine
