!      routines to create probability distributions for dispersal 
!      subroutine neighbor -- investigate neighborhood defined
!         by maximum radius for dispersal, rdst(ii)
!      function xnorm (dist, xmean, sd) -- normal
!      function expon (dist, lam) -- exponential
!      function cauchy(dist, lam) -- Cauchy (fat-tailed)

       subroutine neighbor
       USE mapblock
       USE specblock
!
       implicit none
       integer idst, ii, iii, jjj, il, iu, jl, ju
       real ttl, rcomp, radius, lam
       real xnorm, expon, cauchy
!
!  examine the maximum area enclosed by a circle described by rdst(ii)
       open (12, file='pdf3d.dat',status='unknown')
       do ii = 1, ispecies
          idst = int(rdst(ii) + 0.5)   ! maximum distance for species ii
          il = 1 - idst
          iu = 1 + idst
          jl = 1 - idst
          ju = 1 + idst
          ncnt(ii) = 0
          ttl = 0.0
          rcomp = rdst(ii) * rdst(ii)
          do iii = il,iu
             do jjj = jl,ju
!
                radius = (iii-1)**2 + (jjj-1)**2
!
!  count the number of neighbors in ncnt(ii)
!   record their location in the arrays is and ij
                if (radius .le. rcomp) then
                   ncnt(ii) = ncnt(ii) + 1
                   is(ii,ncnt(ii)) = iii - 1
                   js(ii,ncnt(ii)) = jjj - 1
!
!  choose the appropriate distribution function for distance
!    each function returns the probability of locating that point
!    unif  -- uniform
          select case (dc(ii))
             case ('U')  ! uniform
                dfun(ii,ncnt(ii)) = 1.0
             case ('D')  ! squared inverse distance
                if (radius .gt. 0.0) then
                    dfun(ii,ncnt(ii)) = 1.0 / radius
                  else
                    dfun(ii,ncnt(ii)) = 1.0
                endif
             CASE ('N')  ! norm  -- normal
                radius = sqrt(radius)
                dfun(ii,ncnt(ii)) = xnorm(radius, xbar(ii), sd(ii))
             case ('E')  ! expon -- exponential
                radius = sqrt(radius)
                lam = 1.0 / xbar(ii)
                dfun(ii,ncnt(ii)) = expon(radius, lam)
             case ('C')  ! cauchy -- cauchy distribution
                radius = sqrt(radius)
                lam = max (xbar(ii), sd(ii))
                dfun(ii,ncnt(ii)) = cauchy (radius, lam)
             case default
          end select
!  sum probabilities
                   ttl = dfun(ii,ncnt(ii)) + ttl
                 ENDIF  ! radius < rcomp
             enddo    ! jjj
          enddo       ! iii
!
!  create relative probability probability distribution, dfun
          do iii = 1, ncnt(ii)
             dfun(ii,iii) = dfun(ii,iii) / ttl

     ! print out positive half of distribution
             if (is(ii,iii) .ge. 0 .and. js(ii,iii) .ge. 0) then
     ! ii  - species number
     ! iii - neighbor count
     ! is(ii,iii) - relative row location of neighbor
     ! js(ii,iii) - relative column location of neighbor
     ! dfun(ii,iii) - seed dispersal function
                write (12,100) ii,iii,is(ii,iii),js(ii,iii),dfun(ii,iii)
             endif
100          format (4i8,2x,g12.6) 
          enddo
       enddo          ! ii
!
       close (12)
       return
       end
!
!  normal distribution function
!   probability of moving a distance 'dist' as a function
!   of xmean and sd
       function xnorm (dist, xmean, sd)
       real dist, xmean, sd, x
         x = (dist - xmean) / sd
         xnorm = 0.39894228 * exp(-x*x/2.0)
!        print *, dist, x, xnorm
       return 
       end
!
!  exponential distribution function
!   probability of moving a distance 'dist' as a function of lam
       function expon (dist, lam)
       real lam, dist, expon
          expon = lam * exp (-lam * dist)
!         print *, dist, lam, expon
       return
       end
!
!  cauchy distribution function
!   probability of moving a distance 'dist' as a function of lam
       function cauchy (dist, lam)
       real lam, dist, cauchy
          cauchy = 0.5 * lam / (lam*lam + dist*dist)
!         print *, dist, lam, expon
       return
       end
