!..genmap.f90 contains the following routines for generating random maps
!        subroutine curd_recurse ()
!        Subroutine GenMap
!        Subroutine InputMap
!        Subroutine OutputMap ()
!        Subroutine RandMapX
!        Subroutine RandMapY
!..
       Subroutine GenMap
       USE mapblock
       USE repsblock
!
       INTEGER i, j, ii, level
       REAL xs, prange
!
!..set entire map to zero
       map_hab = 0
!
       select case (maptype)
           case ('R')      !..simple random map -- with replacement
             Call RandMapY
           case ('S')      !..random map -- without replacement
             Call RandMapX
           case ('M')      !..multifractal map
             Call hfract
           case ('G')      !..multifractal map with a gradient
             Call hfract
           case ('I')      !..input map
             Call InputMap
!          case ('C')      !..curdled map
!            level = 1
!            Call curd_recurse (level)
!
       end select
!..generate map classes for random maps based on initial probabilities
       if (maptype .eq. 'R' .or. maptype .eq. 'S') then
          prange = p(ihab) - p(0)
          do i = 2, irow+1
             do j = 2, jcol+1
                if (map_hab(i,j) .gt. 0) then
!                  nxs = int(ran1(iseed_map)*ihab) + 1
                   xs = ran1(iseed_map) * prange + p(0)
                   do ii = 1, ihab
                      if (xs .gt. p(ii-1)) map_hab(i,j) = ii
                   enddo 
                endif
             enddo
          enddo
       endif
!
!..set borders of all maps to -9
3      do i = 1, irow+2
          map_hab(i,1) = -9
          map_hab(i,jcol+2) = -9
       enddo
!
       do j = 1, jcol+2
          map_hab(1,j) = -9
          map_hab(irow+2,j) = -9
       enddo
!
       return
       end
!
!..
       Subroutine OutputMap (iunit)
       USE mapblock
!
       integer iunit, imax, i, j
!  evaluate largest unit in sp_map
       imax = 0
       do i = 2, irow+1
          do j = 2, jcol+1
             IF (map_sp(i,j) .GT. imax) imax = map_sp(i,j)
          end do
       end do
!..output map
       select case (imax)
          case (1:9)
             do i = 2,irow+1
               write(iunit,100) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (10:99)
             do i = 2,irow+1
               write(iunit,101) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (100:999)
             do i = 2,irow+1
               write(iunit,102) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (1000:)
             do i = 2,irow+1
               write(iunit,104) (map_sp(i,j),j=2,jcol+1)
             enddo
       end select
       write (iunit,99)
!
       return
99     format (' ')
100    format (500(i2))
101    format (500(i3))
102    format (500(i4))
104    format (500(i6))
       end
! **
       Subroutine GradMap
!
       USE mapblock
       USE repsblock
!
       pmin = 0.0
       pmax = 1.0
       pinc = (pmax - pmin) / (jcol-1)
       pt = pmin
!
       do j = 2, jcol+1
          do i = 2, irow+1
             if (ran1(iseed_map) .le. pt) map_hab(i,j) = 1
          enddo
          pt = pt + pinc
       enddo
!
       return
       end
!..
       Subroutine RandMapX
!  generate simple random maps
       USE mapblock
       USE repsblock
!
          do i = 2, irow+1
             do j = 2, jcol+1
                y=ran1(iseed_map)
                if (y .le. pr) map_hab(i,j) = 1
             enddo
          enddo
       return
       end
!
!..
       Subroutine RandMapY
!..this routine generates a random map by sampling without replacement
       USE parmlist
       USE mapblock
       USE repsblock
!
       integer non, iset
!
! calculate 'non' -- the number of sites to be "operated" on
       iset = 1
       non = int (pr * irow * jcol + 0.5)
       m_vec = irow * jcol
       if (non .gt. m_vec/2) then
          non = m_vec - non
          iset = 0
          do i = 2, irow+1
             do j = 2, jcol+1
                map_hab(i,j) = 1
             end do
          end do
       end if
       do i = 1, m_vec
          indx(i) = i
       enddo
!
!..generate random numbers
       do k = 1 , non
         ij  = int (ran1(iseed_map) * m_vec) + 1
!..locate within vector
         i = (indx(ij) - 1) / jcol + 1
         j = mod (indx(ij), jcol)
         if (j .eq. 0) j = jcol
         map_hab (i+1,j+1) = iset
!..shift vector down
         do ll = ij, m_vec-1
            indx(ll) = indx(ll+1)
         enddo
         m_vec = m_vec - 1
       enddo
!
       return
       end
!
!..
      Subroutine InputMap
!..assumes map is space delimited, ASCII file
      USE parmlist
      USE mapblock
      USE repsblock
!
!..input map from a data file
      if (irep .eq. 1) open (14, file=mapfile,status='old')
      do i = 2,irow+1
        read (14,*) (map_hab(i,j),j=2,jcol+1)
      enddo
!
      if (irep .eq. nrep) close(14)
!
      return
      end
!
!..
!      RECURSIVE subroutine curd_recurse (level)
!      USE mapblock
!      USE repsblock
!      USE curdblock
!
!      INTEGER level, i, j, k, is, js, non, m_vec
!      INTEGER iprd, jprd, ism, jsm, ij, ll
!
! generate the matrix corresponding to this level
!   by sampling without replacement
!
! calculate 'non' -- the number of sites to be "on"
!      if (pcas(level) .lt. 1.0) then
!         m_vec = ncas(level) * ncas(level)
!         non = int (pcas(level) * m_vec + 0.5)
!         do i = 1, m_vec
!            icdx(i) = i
!         enddo
!  zero out matrix
!         do i = 1, ncas(level)
!            do j = 1, ncas(level)
!               mat(i,j) = 0.0
!            end do
!         end do
!
!..generate random numbers
!         do k = 1, non
!           ij  = int (ran1(iseed_map) * m_vec) + 1
!..locate within vector
!           i = (icdx(ij) - 1) / ncas(level) + 1
!           j = mod (icdx(ij), ncas(level))
!           if (j .eq. 0) j = ncas(level)
!           mat(i,j) = 1
!..shift vector down
!           do ll = ij, m_vec-1
!              icdx(ll) = icdx(ll+1)
!           enddo
!           m_vec = m_vec - 1
!         enddo
!       else
!  if (pcas(level) = 1.0) then set elements of mat(i,j) = 1
!         do i = 1,ncas(level)
!            do j = 1,ncas(level)
!               mat(i,j) = 1
!            end do
!         end do
!      END if
! check for recursive level
!       PRINT 100, level, ((mat(i,j),j=1,2),i=1,2)
!100    format (' level: ', i1,'  mat(i,j) = ',4i4)
!      if (level .lt. icasno) then
! examine matrix and when mat(i,j) = 1, make a recursive call
!        do i = 1, ncas(level)
!           do j = 1, ncas(level)
!             if (mat(i,j) .eq. 1) then
!                isl(level) = i
!                jsl(level) = j
!102      format (' isl,jsl: ',4(2i2,2x))
!                call curd_recurse(level+1)
!             end if
!          end do
!       end do
! done with matrix recursion
!      else
! find locations for placing mat(i,j) into map_hab
!         ism = 1
!         jsm = 1
!         do is = 1, icasno-1
!            iprd = isl(is) - 1
!            jprd = jsl(is) - 1
!            do js = is, icasno-1
!               iprd = iprd * ncas(js)
!               jprd = jprd * ncas(js)
!            end do
!            ism = ism + iprd
!            jsm = jsm + jprd
!         end do
! now put matrix in proper location
!          PRINT 102,(isl(ii),jsl(ii),ii=1,4)
!          PRINT 204,ism,jsm
!204       format (' ism,jsm =',2i5)
!         do i = 1, ncas(level)
!            do j = 1, ncas(level)
!               map_hab(ism+i,jsm+j) = mat(i,j)
!            end do
!         end do
!      end if
!
!      return
!      end subroutine curd_recurse
!
       Subroutine InitSpc
!
       USE repsblock
       USE mapblock
       USE specblock
       INTEGER iplace(0:maxsp), left, right
!
!  set species map borders to zero
       do j = 1,jcol+2
          do i = 1,irow+2
             map_sp(i,j) = 0
          enddo
       enddo
!
!   place species on all optimal sites 
       if (mapinit .eq. 'O') then
!   decide between species for optimum locations
          iplace(0) = 0
          do j = 1, ihab
             if (iopt(1,j) .gt. 0) then
                iplace(j) = 1
                jopt = iopt(1,j)
               else
                iplace(j) = 0
                jopt = 0
             end if

             do k = 2, ispecies
                if (iopt(k,j) .gt. jopt) then
                   iplace(j) = k
                   jopt = iopt(k,j)
                endif
                if (iopt(k,j) .eq. jopt) then
                   if (ran1(iseed_mov) .ge. 0.5) then
                       iplace(j) = k
                       jopt = iopt(k,j)
                   endif
                end if
             end do  ! k loop -- ispecies
          end do     ! j loop -- ihab
!  place species on the map
          do j = 2, jcol+1
             do i = 2, irow+1
                map_sp(i,j) = iplace(map_hab(i,j))
             enddo
          enddo
       endif                ! optimum placement loop
!
!   place species randomly on map edge without regard to habitat type

       if (mapinit .eq. 'L') then
              do i = 2, irow+1
                 map_sp(i,2) = int(ran1(iseed_mov) * ispecies + 1.0)
                 map_sp(i,jcol+1) = int(ran1(iseed_mov) * ispecies + 1.0)
              enddo
       endif
!
!   place species randomly on map without regard to habitat type

       if (mapinit .eq. 'R') then
           do j = 2, jcol+1
              do i = 2, irow+1
                 map_sp(i,j) = int(ran1(iseed_mov) * ispecies + 1.0)
              enddo
           enddo
       endif
!
!   place 2 species on ends of map
       if (mapinit .eq. 'E') then
           left = ispecies / 2
           right = ispecies - left
           do i = 2, irow+1
              map_sp(i,2) = int(ran1(iseed_mov) * left + 1.0)
              map_sp(i,jcol+1) = int(ran1(iseed_mov) * right + left + 1.0)
           enddo
       endif
!
!   place 2 species on 1/2 of map
       if (mapinit .eq. 'H') then
          jhalf = (jcol+2) / 2
          do i = 2, irow+1
             do j = 2, jhalf
                map_sp(i,j) = 1
                map_sp(i,j+jhalf-1) = 2
             enddo
          enddo
       endif
!
       return
       end

