! regrow.f90      LINUX version   7/27/02
!   this version of regrow examines a neighborhood defined    |
!   by 'rdst' and calculates the probability of a seed        |
!   falling on each site according to the UNIFORM, NORMAL,    |
!   EXPONENTIAL or INVERSE SQ distances.  The seed lottery    |
!   is then performed with the probabilities modified by the  |
!   habitat optima and fecundity                              |
!
       subroutine regrow (igen)
       USE repsblock
       USE mapblock
       USE specblock
       USE dstrb_blk
!
       integer :: i, ii, j, jj, iloc, jloc
       real    :: ttl, rx, cfl, cfh, rseed(maxsp), fixt, ds_ratio
!
!  save species distribution in a workfile
       iwk = map_sp
!      print 1111, igen, (ii, vicdf(ii), ii=1,ispecies)
!1111   format (i4,10(i4,1x,f6.4))
!
!  if this is the first generation, find neighborhood to examine
       if (igen .eq. 1) call neighbor
!
!  randomly determine the relative number of "species propagules" 
!    that reach the site -- then use abundance plus suitability rule 
!    to declare winner
       do j = 2, jcol+1
          do i = 2, irow+1
             if (map_hab(i,j) .ne. 0) then
!  check life-form, examine neighborhood and randomly collect propagules
!    and adjust according to species "optimality function"
               mph = map_hab(i,j)
               nsp = map_sp(i,j)
               if (nsp .gt. 0) then    !  consider random site mortality
                  select case (life_form(nsp))
                     case ('A')               ! annuals
                        map_sp(i,j) = 0
                     case ('B')               ! biennials
                        if (ran1(iseed) .lt. 0.5) map_sp(i,j) = 0
                     case ('P')               ! perennials
                        if (ran1(iseed) .lt. 0.1) map_sp(i,j) = 0
                     case ('L')               ! very long-lived
                        if (ran1(iseed) .lt. 0.01) map_sp(i,j) = 0
                     case default
                  END select
               endif

!  now if site is empty 

               if (nsp .eq. 0) then    ! if map location is empty
                  do ii = 1, ispecies
                     rseed(ii) = 0.0
                     if (vicdf(ii) .gt. 0) then  ! if species ii is present
                        do jj = 1, ncnt(ii)
                          iloc = is(ii,jj) + i
                          jloc = js(ii,jj) + j

                          select case (movtype)
                              case ('W') ! wrap edges
                                if (iloc .lt. 2) iloc = irow + iloc
                                if (iloc .gt. irow+1) iloc = iloc - irow
                                if (jloc .lt. 2) jloc = jloc + jcol
                                if (jloc .gt. jcol+1) jloc = jloc - jcol

                              case ('A') ! absorbing rows and columns
                                if (iloc .lt. 2) iloc = 1
                                if (iloc .gt. irow+1) iloc = irow + 2
                                if (jloc .lt. 2) jloc = 1
                                if (jloc .gt. jcol+1) jloc = jcol + 2

                              case ('R') ! reflecting rows and columns
                                if (iloc .lt. 2) iloc = iabs(iloc) + 2
                                if (iloc .gt. irow+1) iloc =  &
                                   (irow+1)-(iloc-irow-2)
                                if (jloc .lt. 2) jloc = iabs(jloc) + 2
                                if (jloc .gt. jcol+1) jloc =  &
                                (jcol+1)-(jloc-jcol-2)
   
                              case default
                          end select
!
! examine probabilities ('dfun') to determine number of propagules
!    must use iwk here because that's the old map
                          if (iwk(iloc,jloc) .eq. ii) then
                             rseed(ii) = rseed(ii) + dfun(ii,jj) * fecund(ii)
                          endif
                       enddo        ! do jj  -- ncnt (neighbors) loop
!
                     end if       ! if species > 0.0


!  SEED RAIN 
!     only allow seed rain when there has been a disturbance
                    if (idefect(igen) .gt. 0) then
           !  make seed rain relative to map abundance
!                      rseed(ii) = rseed(ii) + (sd_rain(ii) * vicdf(ii))
           !  make seed rain absolute
                       rseed(ii) = rseed(ii) + sd_rain(ii) 
                    endif
!
!  multiply by habitat optimum ('iopt') and relative fecundity ('fecund')
                    rseed(ii) = rseed(ii) * iopt(ii,mph) 

!  add disturbance effect as a slope parameter
                    if (freq_type .ne. 'N') then
                       ds_ratio = float(dstb_map(i,j)) / float(max_sevr)
                       rseed(ii) = rseed(ii) - rseed(ii) * (d_slop(ii) * ds_ratio)
                    end if
                 enddo           ! do ii  -- species loop

!
!  calculate total number of propagules for site i,j
                 ttl = 0.0
                 do ii = 1, ispecies 
        !  local extinction if seed source is low
                    if (rseed(ii) .gt. 0.0 .and. rseed(ii) .lt. 0.5) then
                       fixt = exp(-rseed(ii))
                       if (ran1(iseed_mov) .le. fixt) then
                           rseed(ii) = 0.0
                         else
                           ttl = ttl + rseed(ii)
                       endif
                      else       ! rseed(ii) > 1.0 
                       ttl = ttl + rseed(ii)
                    endif
                 enddo           ! do ii -- species
!
!  spin random number and declare winner by calculating the cumulative
!   probabilities "on the fly"
                 if (ttl .gt. 0.0) then
                    rx = ran1(iseed_mov)
                    cfl = 0.0
                    cfh = 0.0
                    do ii = 1, ispecies
                       cfh = cfh + rseed(ii)/ttl
                       if (rx .gt. cfl .and. rx .le. cfh)  &
                          map_sp(i,j) = ii
                       cfl = cfh
                    enddo          ! do ii -- species
                 endif             ! if (ttl .gt. 0.0)
               ENDIF      ! if deadsite
             endif        ! if map_hab > 0
          enddo          ! do i -- rows
       enddo             ! do j -- columns
!
       return
       end
